#ifndef __PCI_NT4_COMMON_H__
#define __PCI_NT4_COMMON_H__

#define NT4_PHYS_MAP_BASE    0x80000000
#define BASE_1MB_SIZE        (1024*1024)
#define BASE_BIOS_OFFSET     0xe0000
#define BASE_BIOS_SIZE       0x20000

#pragma pack(push, 1)

typedef struct _BIOS32_SD {
    UCHAR sig[4];
    ULONG phAddr;
    UCHAR rev;
    UCHAR length;
    UCHAR chksum;
    UCHAR reserved[5];
} BIOS32_SD, *PBIOS32_SD;

#define PCI_BIOS32_SD_SIG    0x49435024

#define PCI_BIOS_PRESENT             0xb101
#define PCI_FIND_DEVICE              0xb102
#define PCI_FIND_CLASS_CODE          0xb103
#define PCI_GENERATE_SPECIAL_CYCLE   0xb106
#define PCI_READ_CONFIG_BYTE         0xb108
#define PCI_READ_CONFIG_WORD         0xb109
#define PCI_READ_CONFIG_DWORD        0xb10a
#define PCI_WRITE_CONFIG_BYTE        0xb10b
#define PCI_WRITE_CONFIG_WORD        0xb10c
#define PCI_WRITE_CONFIG_DWORD       0xb10d
#define PCI_GET_IRQ_ROUTING_OPTIONS  0xb10e
#define PCI_SET_PCI_IRQ              0xb10f


typedef struct _PCI_BIOS32_IRQ_ROUTING_TABLE_ENTRY {
    UCHAR BusId;
    UCHAR DeviceNumber;

    struct {
        UCHAR  Link;
        USHORT IRQBitMap;
    } INTA;

    struct {
        UCHAR  Link;
        USHORT IRQBitMap;
    } INTB;

    struct {
        UCHAR  Link;
        USHORT IRQBitMap;
    } INTC;

    struct {
        UCHAR  Link;
        USHORT IRQBitMap;
    } INTD;
    
    UCHAR SlotNumber;
    UCHAR Reserved;

} PCI_BIOS32_IRQ_ROUTING_TABLE_ENTRY, *PPCI_BIOS32_IRQ_ROUTING_TABLE_ENTRY;

typedef struct _IRQ_ROUTING_OPTIONS {
    USHORT BufferSize;
    PPCI_BIOS32_IRQ_ROUTING_TABLE_ENTRY Addr;
} IRQ_ROUTING_OPTIONS, *PIRQ_ROUTING_OPTIONS;

#ifndef USER_MODE

extern ULONG BIOS_ver;
extern PVOID BIOS_VA;
extern PVOID PCI_BIOS;

NTSTATUS
PciBios32_Init();

PBIOS32_SD
Check_BIOS32_SD(
    ULONG phAddr
    );

ULONG
PciIsBIOSPresent();

PPCI_BIOS32_IRQ_ROUTING_TABLE_ENTRY
PciGetIrqRoutingTable(
    PULONG  pItemCount,
    PULONG  pIrqBitMap
    );

VOID
PciBios32_Deinit();

#endif //USER_MODE

#pragma pack(pop, 1)

#endif //__PCI_NT4_COMMON_H__
